#
# 11Dec1999	David McKee
#	Make the conversion string for "|"s a variable (PC).
#	Fix bug that was ditching all the empty lines, by introducing HaveLast.
#	More generalization on the part-name identification.
#
# 8Dec1999	David McKee
#	Convert all source "|"s to ":"s due to a bug in ObiWan.
#
# 7Dec1999 	David McKee
#	Bug fixes to the script. The identification of the part name was broken,
#	because it assumed a line of the form "<name><whitespace>#<comment>' Make
#	this more robust.
#
# 28Nov19998	David McKee
#	Inital implementation.
#
#
# A prototype of the awk script used by the Leia's Plea script.
#
#
# I always keep one line in a buffer (LastLine) to optimise the use of the
# continuation.

# Set things up.
BEGIN {
	NumLines = 0;			# The number of lines already printed in the "topic".
	LastLine = ""			# Buffer holding the last line of help text read.
	HaveLast = 0;			# Is there a real line in the buffer.
	Name= "";				# Root name of the current topic.
	PartNum = 1;			# Which part (of a multi-part topic) are we in.
	
	HaveName = 0;			# Flag tells if we have the name of this topic.
	
	# Set the default maximun number of lines and pipe replacement string...
	if ( MaxLines == 0 ) { MaxLines = 10 };
	if ( PC == 0 ) { PC = ":" };
}

# When we encounter a divider...
/^-$/ {
	# If the lastline buffer is not empty, finish writing the last entry...
	if ( If HaveLast && ( LastLine != "" ) ) {
		if ( NumLines >= MaxLines ) {  # Panic! This should never happen!
			exit(1);
		}
		printf "%s|\n",LastLine;
	} else { 
		printf "|\n" 
	};
	NumLines=0;
	LastLine="";
	HaveLast=0;
	Name="";
	PartNum=1;
	HaveName=0;
	
	next;
}

# Other Lines...
{
	# If we don't have the name, parse this line to get it.
	if ( !HaveName ) {
		# Assume everything up to '<whitespace>[#-]' is the name
		# If there is no '[#-]', try everything up to '<whitespace><EOL>',
		# And if that fails, use just the whole line.
		if ( match($0,"[ 	]*[#-]") ) {
			Name=clean(substr($0,0,RSTART-1));
		} else if ( match($0,"[ 	]*$") ) {
			Name=clean(substr($0,0,RSTART-1));
		} else {
			Name=clean($0);
		}
		PartNum=1;
		HaveName=1;
	}

	# If we are going to overflow the allowed size, skip to the next "topic"
	if ( NumLines >= (MaxLines-1) ) {
		PartNum++;
		NumLines = 1;
		print "[continues...]|"
		printf "%s,part%2d\n",Name,PartNum;
	}
	
	# Print the last line, and move the current line into the buffer.
	if ( HaveLast ) {
		print LastLine;
		NumLines++;
	}
	#LastLine=$0
	LastLine=clean($0);
	HaveLast=1;
	
	next;
}


# Clean up after ourselves...
END {
	# Write out any remaining line...just like for /^-$/
	if ( HaveLast && ( LastLine != "" ) ) {
		if ( NumLines >= MaxLines ) {  # Panic! This should never happen!
			exit(1);
		}
		printf "%s|\n",LastLine;
	} else {
		printf "|\n";
	}

}


# Function clean is supposed to make any changes to the source text
# that are need to conform with the limits of the output format.
#
# In this case, "|" is special significant in the output, so I replace it
# with PC.
function clean(source) {
	gsub("[|]",PC,source);
	return source;
}